#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
from osgeo import gdal
from qgis.core import *
from qgis.PyQt.QtGui import QFont, QColor
import shutil
from common import safe_remove

def createQGISMap(tiffile, pngfile, qgs_template, items_dict, colorbarfile, level=3, et='', mapid='Map1', removetiff=True):
    print(tiffile, pngfile, qgs_template, items_dict, colorbarfile)
    # {{{
    # 解决ERROR 6: The PNG driver does not support update access to existing datasets.
    gdal.PushErrorHandler('CPLQuietErrorHandler')
    gdal.SetConfigOption("GDAL_PAM_ENABLED", "NO")

    # create a reference to the QgsApplication, setting the # second argument to False disables the GUI
    qgs = QgsApplication([], False)
    # load providers
    qgs.initQgis()

    # p1：读取qgs模板文件
    project = QgsProject.instance()
    project.read(qgs_template)

    # 加载栅格图层
    rlayer = QgsRasterLayer(tiffile, "rlayer")
    project.addMapLayer(rlayer)

    # 修改图层顺序
    root = project.layerTreeRoot()

    myrlayer = root.findLayer(rlayer.id())
    rlayclone = myrlayer.clone()
    parent = myrlayer.parent()
    parent.insertChildNode(level, rlayclone)
    parent.removeChildNode(myrlayer)

    # rlayer.renderer().setRedBand(3)
    # rlayer.renderer().setGreenBand(2)
    # rlayer.renderer().setBlueBand(1)
    ContrastEnhancement = QgsContrastEnhancement.StretchToMinimumMaximum

    myRedBand = rlayer.renderer().redBand()
    myRedType = rlayer.renderer().dataType(myRedBand)
    myRedEnhancement = QgsContrastEnhancement(myRedType)
    myRedEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
    myRedEnhancement.setMinimumValue(0)
    myRedEnhancement.setMaximumValue(255)
    rlayer.renderer().setRedContrastEnhancement(myRedEnhancement)

    myGreenBand = rlayer.renderer().greenBand()
    myGreenType = rlayer.renderer().dataType(myGreenBand)
    myGreenEnhancement = QgsContrastEnhancement(myGreenType)
    myGreenEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
    myGreenEnhancement.setMinimumValue(0)
    myGreenEnhancement.setMaximumValue(255)
    rlayer.renderer().setGreenContrastEnhancement(myGreenEnhancement)

    myBlueBand = rlayer.renderer().blueBand()
    myBlueType = rlayer.renderer().dataType(myBlueBand)
    myBlueEnhancement = QgsContrastEnhancement(myBlueType)
    myBlueEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
    myBlueEnhancement.setMinimumValue(0)
    myBlueEnhancement.setMaximumValue(255)
    rlayer.renderer().setBlueContrastEnhancement(myBlueEnhancement)
    rlayer.triggerRepaint()

    # 获取qgs模板文件中的制图模板
    layout = project.layoutManager().layoutByName("layout")
    if str(type(et)) ==  str(type('')):
        pass
    else:
        mapitem = layout.itemById(mapid)
        extent = QgsRectangle(et[0], et[1], et[2], et[3])
        extent.scale(1.0)
        mapitem.zoomToExtent(extent)

    if colorbarfile == '':
        pass
    else:
        try:
            rec = layout.itemById("colorbar")
            rec.setPicturePath(colorbarfile)    
        except Exception as e:
            print(e)

    for k, v in items_dict.items():
        print(k, v)
        try:
            if re.search('colorbar', k):
                rec = layout.itemById(k)
                rec.setPicturePath(v)    
            else:
                layout.itemById(k).setText(v)
        except Exception as e:
            print(e)

    layout.refresh()

    # 输出图片
    exporter = QgsLayoutExporter(layout)
    exporter.exportToImage(pngfile+'.tmp.qgs.png', QgsLayoutExporter.ImageExportSettings())
    print(tiffile, pngfile, qgs_template, items_dict, colorbarfile)
    print(removetiff)
    if removetiff:
        safe_remove(tiffile)

    if re.search(r'\.cliptiff\.', tiffile):
        safe_remove(tiffile)

    os.replace(pngfile+'.tmp.qgs.png', pngfile)
    return
    # }}}


def createQGISMaps(tiffiles, pngfile, qgs_template, items_dict, colorbarfile, levels='3,5', et='', mapid='Map1', removetiff=True):
    # {{{
    # 解决ERROR 6: The PNG driver does not support update access to existing datasets.
    gdal.PushErrorHandler('CPLQuietErrorHandler')
    gdal.SetConfigOption("GDAL_PAM_ENABLED", "NO")

    # create a reference to the QgsApplication, setting the # second argument to False disables the GUI
    qgs = QgsApplication([], False)
    # load providers
    qgs.initQgis()

    # p1：读取qgs模板文件
    project = QgsProject.instance()
    project.read(qgs_template)

    itiff = -1
    levels = re.findall(r'\d+', str(levels))
    for tiffile in tiffiles:
        itiff += 1
        level = int(levels[itiff])
        # 加载栅格图层
        rlayer = QgsRasterLayer(tiffile, "rlayer")
        project.addMapLayer(rlayer)

        # 修改图层顺序
        root = project.layerTreeRoot()

        myrlayer = root.findLayer(rlayer.id())
        rlayclone = myrlayer.clone()
        parent = myrlayer.parent()
        parent.insertChildNode(level, rlayclone)
        parent.removeChildNode(myrlayer)

        # rlayer.renderer().setRedBand(3)
        # rlayer.renderer().setGreenBand(2)
        # rlayer.renderer().setBlueBand(1)
        ContrastEnhancement = QgsContrastEnhancement.StretchToMinimumMaximum

        myRedBand = rlayer.renderer().redBand()
        myRedType = rlayer.renderer().dataType(myRedBand)
        myRedEnhancement = QgsContrastEnhancement(myRedType)
        myRedEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
        myRedEnhancement.setMinimumValue(0)
        myRedEnhancement.setMaximumValue(255)
        rlayer.renderer().setRedContrastEnhancement(myRedEnhancement)

        myGreenBand = rlayer.renderer().greenBand()
        myGreenType = rlayer.renderer().dataType(myGreenBand)
        myGreenEnhancement = QgsContrastEnhancement(myGreenType)
        myGreenEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
        myGreenEnhancement.setMinimumValue(0)
        myGreenEnhancement.setMaximumValue(255)
        rlayer.renderer().setGreenContrastEnhancement(myGreenEnhancement)

        myBlueBand = rlayer.renderer().blueBand()
        myBlueType = rlayer.renderer().dataType(myBlueBand)
        myBlueEnhancement = QgsContrastEnhancement(myBlueType)
        myBlueEnhancement.setContrastEnhancementAlgorithm(ContrastEnhancement, True)
        myBlueEnhancement.setMinimumValue(0)
        myBlueEnhancement.setMaximumValue(255)
        rlayer.renderer().setBlueContrastEnhancement(myBlueEnhancement)
        rlayer.triggerRepaint()

    # 获取qgs模板文件中的制图模板
    layout = project.layoutManager().layoutByName("layout")
    if str(type(et)) ==  str(type('')):
        pass
    else:
        mapitem = layout.itemById(mapid)
        extent = QgsRectangle(et[0], et[1], et[2], et[3])
        extent.scale(1.0)
        mapitem.zoomToExtent(extent)

    if colorbarfile == '':
        pass
    else:
        rec = layout.itemById("colorbar")
        rec.setPicturePath(colorbarfile)    

    for k, v in items_dict.items():
        if re.search('colorbar', k):
            rec = layout.itemById(k)
            rec.setPicturePath(v)    
        else:
            layout.itemById(k).setText(v)
    layout.refresh()

    # 输出图片
    exporter = QgsLayoutExporter(layout)
    exporter.exportToImage(pngfile+'.tmp.png', QgsLayoutExporter.ImageExportSettings())
    if removetiff:
        pass
    os.replace(pngfile + '.tmp.png', pngfile)
    # }}}


if __name__ == '__main__':
    tiffile = r"D:\bin\peng\amv.test.tiff"
    pngfile = r"D:\bin\peng\amv1.png"
    qgs_template = r"D:\bin\peng\china\template.qgs"

    chndict = {"title": "卫星云导风产品空间分布图",
               "date": "2021-01-11 10:10:00(北京时)",
               "satellite": "卫星/传感器:FY-4A/AGRI",
               "resolution": "分辨率:4000M"}

    createQGISMap(tiffile, pngfile, qgs_template, chndict, pngfile)



import re
from .b64save import json2dict, dict2json
import os
import sys
from . import time_htht as htt
import time
import shutil


displayed_lines = []
tmp_datas = {}
tmp_datas['term_height'] = 0
tmp_datas['status'] = ''
tmp_datas['simple'] = not(os.isatty(sys.stdout.fileno()))


def get_terminal_height():
    _, term_height_new = shutil.get_terminal_size()
    term_height = tmp_datas['term_height']
    t = term_height_new == term_height
    tmp_datas['term_height'] = term_height_new
    if t:
        pass
    else:
        clear_screen()
        # print_status_best(tmp_datas['status'])
        # p = tmp_datas['status']
        # sys.stdout.write(f"\033[{term_height_new};0H{p}\033[K")
        # sys.stdout.flush()
        pass

    return term_height_new


class Colors:
    RESET = "\033[0m"  # 重置颜色
    BLACK = "\033[30m"   # 红色
    RED = "\033[31m"   # 红色
    GREEN = "\033[32m" # 绿色
    YELLOW = "\033[33m" # 黄色
    BLUE = "\033[34m"  # 蓝色
    CYAN = "\033[36m"  # 青色
    WHITE = "\033[37m"  # 青色
    BOLD = "\033[1m"   # 加粗
    UNDERLINE = "\033[4m" # 下划线


def dtime2stime(dtime):
    dtime = str(dtime)
    dtime = re.sub('(^0\.0*\d{3}).*', r'\1', dtime)
    dtime = re.sub(r'(.*[^0].*\.\d{2})\d+', r'\1', dtime)
    dtime = re.sub(r'(\..*[^0])0+$', r'\1', dtime)
    stime = re.sub(r'\.0+$', '.0', dtime)
    return stime


def gen_process_tab(ns, nsl, type='low'):
    fn = int(ns*40/nsl)
    hn = 40 - fn
    if False:
        fn -= 1
        stmp = '['+'='*fn+'>'+'.'*hn+']'
    if False:
        stmp = '['+'#'*fn+''+'.'*hn+']'
    if type == 'high':
        stmp = f"{Colors.WHITE}"+'━'*fn+f"{Colors.RESET}"+\
			   f"{Colors.RED}"+'━'*hn+f"{Colors.RESET}"
    if type == 'low':
        stmp = '['+\
				f"{Colors.RED}{Colors.BOLD}"+'#'*fn+f"{Colors.RESET}"+\
				f"{Colors.WHITE}"+'.'*hn+f"{Colors.RESET}"+\
				']'
    stmp += ' '+str(int(ns/nsl*100))+'% ('+str(int(ns))+'/'+str(nsl)+')'
    return stmp


def clear_screen():
    """清屏并将光标移至顶部"""
    sys.stdout.write("\033[2J\033[H")
    sys.stdout.flush()


def print_status(progress):
    """在终端底部打印进度状态"""
    term_height = get_terminal_height()
    sys.stdout.write(f"\033[{term_height};0HProgress: {progress}%\033[K")  # 定位到最后一行
    sys.stdout.flush()


def print_status_best(progress):
    """在终端底部打印进度状态"""
    if tmp_datas['simple']:
        print(progress, flush=True)
        return

    tmp_datas['status'] = progress
    term_height = get_terminal_height()
    sys.stdout.write(f"\033[{term_height};0H{progress}\033[K")
    sys.stdout.flush()


def print_output(line):
    """打印滚动输出内容"""
    if tmp_datas['simple']:
        print(line, flush=True)
        return

    sys.stdout.write("\033[H")  # 光标回到第一行
    displayed_lines.append(line)
    term_height = get_terminal_height()
    while len(displayed_lines) > term_height-1:
        displayed_lines.pop(0)
    for line in displayed_lines:
        sys.stdout.write(f"{line}\n")
    sys.stdout.flush()


def project(*args, **kw):
    p = PROJECT(*args, **kw)
    return p.say, p


class PROJECT():

    """Docstring for project. """

    def __init__(self, inputjson, steps='初始化|输出',
                 pname='', prodir='', processtab='low',
                 simple=True):
        tmp_datas['simple'] = simple
        """TODO: to be defined. """
        self.processtab=processtab
        if pname == '':
            self.pname = 'tctb'
        else:
            self.pname = pname
        clear_screen()

        self.status = '成功'
        self.error = ''
        self.outputs = []
        self.time0 = time.time()
        self.outdir = '/tmp/'
        self.time00 = self.time0
        if prodir == '':
            self.prodir = os.path.dirname(
                    os.path.abspath(__file__))
        else:
            self.prodir = prodir

        if str(type(steps)) == str(type('-')):
            self.steps = re.split(r'\|', steps)
        else:
            self.steps = steps

        self.status_tab = gen_process_tab(0, len(self.steps), self.processtab)
        self.progress = [0, len(self.steps)]

        try:
            self.input = json2dict(inputjson)
        except Exception as e:
            if pname == '':
                self.pname = 'base'
            else:
                self.pname = pname
            inputjson = self.prodir+'/'+self.pname+'/input.json'
            self.input = json2dict(inputjson)

        try:
            settingsjson = self.prodir+'/'+\
                    self.pname+'/settings.json'
            self.settings = json2dict(settingsjson)
        except Exception as e:
            self.settings = {'info':''}

        try:
            self.outdir = self.input['resultPath']
            self.pfile = self.input['primaryFile']
        except Exception as e:
            pass

        self.flow_file = self.input['resultFlowFile']
        self.result_file = self.input['resultJsonFile']
        self.log_file = self.input['resultLogFile']

        # 初始化flowjson
        self.flow_dict = json2dict(self.prodir+'/'+self.pname+'/flow.json')
        self.step_dict = {} 
        num = 0
        for sname in self.steps:
            num += 1
            tmp = json2dict(self.prodir+'/'+self.pname+'/step.json')
            tmp['stepName'] = sname
            tmp['stepNo'] = str(num)
            self.flow_dict['step'].append(tmp)
            self.step_dict[sname] = tmp
        dict2json(self.flow_dict, self.flow_file)
        self.flow('初始化')

        # 初始化resultjson
        self.result_dict = json2dict(self.prodir+'/'+self.pname+'/result.json')
        self.result_dict['productionTime'] =\
                htt.time2str(time.time()+8*3600, 'yyyy/mm/dd HH:MM:SS CST')
        dict2json(self.result_dict, self.result_file)

    def flow(self, sname):
        if sname in self.step_dict:
            pass
        else:
            return

        if self.step_dict[sname]['status'] == 0:
            return

        ns = 0
        for s in self.steps:
            ns += 1
            if self.step_dict[s]['status'] == 0:
                continue
            stime = htt.time2str(time.time()+8*3600, 'yyyy/mm/dd HH:MM:SS CST')

            # 处理完成时间字符串
            dtime = str(time.time()-self.time0)
            dtime = re.sub('(^0\.0*\d{3}).*', r'\1', dtime)
            dtime = re.sub(r'(.*[^0].*\.\d{2})\d+', r'\1', dtime)
            dtime = re.sub(r'(\..*[^0])0+$', r'\1', dtime)
            dtime = re.sub(r'\.0+$', '.0', dtime)

            self.step_dict[s]['status'] = 0 
            self.step_dict[s]['log'] =\
                    s+'完成, 耗时'+dtime+'秒' 
            self.step_dict[s]['timeStamp'] = stime
            dict2json(self.flow_dict, self.flow_file)

            self.log(s+'完成')
            self.time0 = time.time()
            if s == sname:
                break

        self.progress[0] = ns
        stmp = gen_process_tab(ns, len(self.steps), 
                               type=self.processtab)
        print_status_best(stmp)
        self.status_tab = stmp

    def finish(self, info={}):
        if self.error == '':
            pass
        else:
            self.status = '失败'

        if self.status == '失败':
            self.result_dict['status'] = '1'
            self.result_dict['message'] = self.error
        if self.status == '成功':
            self.result_dict['status'] = '0'
            self.result_dict['message'] = '执行成功。'
            self.flow('输出')
        if self.status == '重做':
            self.result_dict['status'] = '9'
            self.result_dict['message'] =\
                    '需要重做。'+self.error
        dict2json(self.result_dict, self.result_file)
        return

    def result(self, outpath='', info={}):
        if outpath == '':
            if self.error == '':
                pass
            else:
                self.status = '失败'

            if self.status == '失败':
                self.result_dict['status'] = '1'
                self.result_dict['message'] = self.error
            if self.status == '成功':
                self.result_dict['status'] = '0'
                self.result_dict['message'] = '执行成功。'
                self.flow('输出')
            if self.status == '重做':
                self.result_dict['status'] = '9'
                self.result_dict['message'] =\
                        '需要重做。'+self.error
            dict2json(self.result_dict, self.result_file)
            self.log('运行完毕, 耗时'+\
                dtime2stime(time.time()-self.time00)+'秒')
            return

        self.flow('处理')
        product = json2dict(self.prodir+'/'+
                        self.pname+'/product.json')

        if str(type(info)) == str(type('')):
            info1 = {}
            for tmp in re.split(r'\|', info):
                try:
                    tmp1 = re.search('(.+):(.+)', tmp)
                    info1[tmp1.group(1)] = tmp1.group(2)
                except Exception as e:
                    pass
            info = info1
    
        self.outputs.append(outpath)
        if re.search(r'^:', outpath):
            outpath = re.sub(r'^:'+self.outdir, '', outpath)
        if re.search(r'^\|', outpath):
            outpath = re.sub(r'^\|'+self.outdir, '', outpath)
        if re.search(r'-norootpath-', self.settings['info']):
            outpath = re.sub(r'^'+self.outdir, '', outpath)

        product['filePath'] = outpath
        for k in product.keys():
            if k in info:
                product[k] = info[k]
        self.result_dict['result'].append(product)
        dict2json(self.result_dict, self.result_file)

    def log(self, stmp):
        stmp0 = stmp

        if stmp == '输出开始':
            self.flow('处理')

        if re.search(r'.*开始$', stmp):
            self.time0 = time.time()
            # print_output('/*----------------------------')

        if re.search(r'.*完成$', stmp):
            stime = dtime2stime(time.time()-self.time0)
            stmp = stmp + ', 耗时'+stime+'秒'
            self.time0 = time.time()

        with open(self.log_file, 'a') as f:
            s = htt.time2str(time.time()+8*3600, 
                             'yyyy/mm/dd HH:MM:SS CST: ')+stmp
            print_output(s)
            f.write(s+'\n')

        if re.search(r'.*完成$', stmp0):
            # print_output('----------------------------*/\n')
            pass
        
        if tmp_datas['simple']:
            return

        ns = self.progress[0]
        ns += (int(ns)+1-ns)/3
        if ns > self.progress[1]:
            ns = self.progress[1]
        self.progress[0] = ns
        stmp = gen_process_tab(ns, len(self.steps), 
                       type=self.processtab)
        print_status_best(stmp)
        self.status_tab = stmp

    def update(self):
        return

    def say(self, stmp='', info={}):

        if re.search('建立新项目:', stmp):
            outputpath = re.search(r':(.+)', stmp).group(1)
            os.system('cp -r '+self.prodir+'/tctb '+outputpath)

        if stmp == '更新结果状态' or\
                stmp == '' or stmp == 'update result':
            self.result()
            return

        if re.search(r'^错误:', stmp):
            print_output(stmp)
            self.error = stmp

        if re.search(r'^ERROR:', stmp):
            print_output(stmp)
            self.error = stmp

        if re.search(r'^error:', stmp):
            print_output(stmp)
            self.error = stmp

        if re.search(r'.+完成$', stmp):
            step = re.search(r'(.+)完成$', stmp).group(1)
            if step in self.step_dict:
                self.flow(step)
                return

        if re.search(r'^[p|o|r]?:', stmp):
            outfile = re.sub(r'^[p|o|r]?:', '', stmp)
            self.result(outfile, info)
            self.log('输出文件: '+outfile)
            return
 
        if re.search(r'^output:', stmp):
            outfile = re.sub(r'^output:', '', stmp)
            self.result(outfile, info)
            self.log('输出文件: '+outfile)
            return

        if re.search(r'^file:', stmp):
            outfile = re.sub(r'^output:', '', stmp)
            self.result(outfile, info)
            self.log('输出文件: '+outfile)
            return

        if re.search(r'^result:', stmp):
            outfile = re.sub(r'^result:', '', stmp)
            self.result(outfile, info)
            self.log('输出文件: '+outfile)
            return

        if re.search(r'^输出:', stmp):
            outfile = re.sub(r'^输出:', '', stmp)
            self.result(outfile, info)
            self.log('输出文件: '+outfile)
            return

        if stmp == 'INPUT' or\
                stmp == 'input' or\
                stmp == '显示输入:' or\
                stmp == '输入': 
            try:
                self.log('输入主文件: '+self.pfile)
            except Exception as e:
                pass
            try:
                self.log('输出根目录: '+self.outdir)
            except Exception as e:
                pass
            return
        self.log(stmp)



import time
import time_htht as htt
print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
import sys
import re
import argparse
import os

print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))

from png2qgis import fig

def parse_args():
    # {{{
    """
    Parse input arguments
    """
    parser = argparse.ArgumentParser(description='FY3D conv')
    parser.add_argument('--png', dest='png',
                        help='input png file',
                        default=None, type=str)
    parser.add_argument('--qgs', dest='qgs',
                        help='qgs template',
                        default=None, type=str)
    parser.add_argument('--png_json', dest='png_json',
                        help='png json',
                        default=None, type=str)
    parser.add_argument('--qgs_json', dest='qgs_json',
                        help='qgs json',
                        default=None, type=str)

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    args = parser.parse_args()
    return args
    # }}}

# 获取参数
args = parse_args()

f = fig()
f.pngfile = args.png
f.qgs_template = args.qgs
if args.qgs_json is None:
    f.qgs_json_file = f.qgs_template+'.json'
else:
    f.qgs_json_file = args.qgs_json

if args.png_json is None:
    f.png_json_file = f.pngfile+'.json'
else:
    f.png_json_file = args.png_json

if re.search(',', f.pngfile):
    f.pngs2qgs_json()
else:
    f.png2qgs_json()




import sys
import time


try:
    from projects import project
    say, p = project(sys.argv[-1], steps='输入|计算|输出')
except Exception as e:
    print(e)
    say = print 


def main():
    say('输入开始')
    time.sleep(0.1)
    infile = '/'
    outdir = '/tmp/'

    try:
        infile = p.pfile
        outdir = p.outdir
        say('输入')
    except Exception as e:
        pass
    say('输入完成')

    say('计算开始')
    time.sleep(1)
    say('计算完成')
    

    say('绘图开始')
    time.sleep(0.625)
    say('绘图完成')
 
    say('输出开始')
    time.sleep(0.412)
    say(':/tmp/test1.png')
    say(':/tmp/test2.png')
    say('输出完成')


if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        say('error:'+str(e))
    say()



#!/bin/python3
import time as tm
import re
import datetime


def time2str(t,ss, num=1): # {{{
    st = str(datetime.datetime.utcfromtimestamp(t))
    # st = tm.strftime("%Y-%m-%d %H:%M:%S", tm.gmtime(t + tm.mktime((1970, 1, 1, 0, 0, 0, 0, 0, 0))))
    time = re.search(r'(\d\d\d\d)\-(\d\d)\-(\d\d) (\d\d)\:(\d\d)\:(\d\d)',st)
    year = time.group(1)
    year2 = re.search(r'\d\d(\d\d)',year)
    year2 = year2.group(1)
    month = time.group(2)
    dom = time.group(3)
    doy = vec2time(year, month, dom, 0, 0, 1) - vec2time(year, 1, 0, 0, 0, 0)
    doy = round(doy/24/3600)+1000
    doy = re.sub(r'^1', '', str(doy))
    hour = time.group(4)
    minute = time.group(5)
    sec = time.group(6)
    st = ss
    m3 = {'01':'Jan', '02':'Feb', '03':'Mar', '04':'Apr', 
          '05':'May', '06':'Jun', '07':'Jul', '08':'Aug', 
          '09':'Sep', '10':'Oct', '11':'Nov', '12':'Dec'}
    for i in range(0, num):
        st = re.sub('yyyy',year,st);
        st = re.sub('mmm',m3[month],st);
        st = re.sub('mm',month,st);
        st = re.sub('ddd',doy,st);
        st = re.sub('dd',dom,st);
        st = re.sub('HH',hour,st);
        st = re.sub('MM',minute,st);
        st = re.sub('SS',sec,st);
    return st # }}}


def time2vec(t): # {{{
    st = datetime.datetime.utcfromtimestamp(t)
    st = str(st)
    time = re.search(r'(\d\d\d\d)-(\d\d)-(\d\d) (\d\d)\:(\d\d)\:(\d\d)',st)
    year = int(time.group(1))
    month = int(time.group(2))
    dom = int(time.group(3))
    hour = int(time.group(4))
    minute = int(time.group(5))
    sec = int(time.group(6))
    return year,month,dom,hour,minute,sec # }}}


def str2time(s, rs='yyyymmddHHMMSS'):  # {{{
    if rs == 'yyyymmddHHMMSS':
        try:
            s = re.search(r'(\d{14})', s).group(1)
        except Exception as e:
            pass
        rt = re.search(
            r'(\d\d\d\d)[^\d]*(\d\d)[^\d]*(\d\d)[^\d]*' +
            r'(\d\d)[^\d]*(\d\d)[^\d]*(\d\d)', s)
        year = rt.group(1)
        month = rt.group(2)
        dom = rt.group(3)
        hour = rt.group(4)
        minute = rt.group(5)
        sec = rt.group(6)
    else:
        t = re.search(r'yyyy', rs)
        year = s[t.span(0)[0]:t.span(0)[1]]
        t = re.search(r'mm', rs)
        month = s[t.span(0)[0]:t.span(0)[1]]
        t = re.search(r'dd', rs)
        dom = s[t.span(0)[0]:t.span(0)[1]]
        t = re.search(r'HH', rs)
        hour = s[t.span(0)[0]:t.span(0)[1]]
        t = re.search(r'MM', rs)
        minute = s[t.span(0)[0]:t.span(0)[1]]
        t = re.search(r'SS', rs)
        sec = s[t.span(0)[0]:t.span(0)[1]]

    if False:
        print(year,month,dom,hour,minute,sec)
        ts = tm.mktime((int(year), int(month), int(dom),
                        int(hour), int(minute), int(sec), 0, 0, 0)) - \
            tm.mktime((1970, 1, 1, 0, 0, 0, 0, 0, 0))

        print(ts)
    # 创建datetime对象
    dt = datetime.datetime(year=int(year), month=int(month), day=int(dom),
                        hour=int(hour), minute=int(minute), second=int(sec))

    # 计算与1970年1月1日午夜UTC的时间差
    epoch = datetime.datetime.utcfromtimestamp(0)
    delta = dt - epoch

    # 将时间差转换为秒数
    ts = delta.total_seconds()
    return ts  # }}}


def vec2time(year, month, dom, hour, minute, sec):  # {{{
    ts = tm.mktime((int(year), int(month), int(dom), int(hour), int(minute), int(sec), 0, 0, 0)) - \
    tm.mktime((1970, 1, 1, 0, 0, 0, 0, 0, 0))
    return ts  # }}}


if __name__ == "__main__":
    infile = '/tmp//FY4/FY4B/GIIRS/L2/AVP/REGC/2024/2024080700/FY4B-_GIIRS-_N_REGC_1050E_L2-_AVP-_MULT_NUL_20240807080000_20240807080000_012KM_V0001.NC'
    # infile = '2024/08/07-08:00:00'
    print(infile)
    dt = str2time(infile)
    print(time2str(dt, 'yyyy/mm/dd HH:MM:SS'))



import numpy as np
import json
import base64
from io import BytesIO
import re
import os

def dict2b64(data):
    if str(type(data)) == str(type({0:1})):
        ks = list(data.keys())
        for k in ks:
            if str(type(data[k])) == str(type(np.array([0,1]))):
                b = base64.b64encode(data[k])
                sb = str(b)[2:-1]+'|numpy'
                data[k+'_type'] = str(data[k].dtype)
                data[k+'_shape'] = data[k].shape
                data[k] = sb
            else:
                dict2b64(data[k])
    elif str(type(data)) == str(type([0,1])):
        for k in range(0, len(data)):
            dict2b64(data[k])
    else:
        return


def dict2numpy(data, file, name):
    if str(type(data)) == str(type({0:1})):
        ks = list(data.keys())
        for k in ks:
            if str(type(data[k])) == str(type(np.array([0,1]))):
                outdir = file+'.data/'
                if ~os.path.exists(outdir):
                    os.system('mkdir -p '+outdir)
                b = name+'.'+k+'.npy'
                np.save(outdir+b, data[k])
                data[k] = b
            else:
                dict2numpy(data[k],file,name+'.'+str(k))
    elif str(type(data)) == str(type([0,1])):
        for k in range(0, len(data)):
            dict2numpy(data[k],file,name+'.'+str(k))
    else:
        return


def b642dict(data):
    if str(type(data)) == str(type({0:1})):
        for k in data.keys():
            if str(type(data[k])) == str(type('s')):
                if re.search(r'\|numpy$', data[k]):
                    b = base64.decodebytes(bytes(re.sub(r'\|.*','',data[k]), "utf8"))
                    data[k] = np.frombuffer(b, dtype=data[k+'_type']).reshape(data[k+'_shape'])
            else:
                b642dict(data[k])
    elif str(type(data)) == str(type([0,1])):
        for k in range(0, len(data)):
            b642dict(data[k])
    else:
        return


def numpy2dict(data,file,name=''):
    if str(type(data)) == str(type({0:1})):
        for k in data.keys():
            if str(type(data[k])) == str(type('s')):
                if re.search(r'\.npy$', data[k]):
                    if re.search(r'ALLDATA', name):
                        print(file+'.data/'+data[k])
                        data[k] = np.load(file+'.data/'+data[k])
                    else:
                        if re.search(r'0\.'+name+r'\.npy$', data[k]):
                            data[k] = np.load(file+'.data/'+data[k])
            else:
                numpy2dict(data[k],file,name)
    elif str(type(data)) == str(type([0,1])):
        for k in range(0, len(data)):
            numpy2dict(data[k],file,name)
    else:
        return


def dict2json(data, file):
    dict2b64(data)
    with open(file, "w", encoding='utf-8') as f:
        json.dump(data, f)


def dict2jsons(data, file):
    dict2numpy(data, file, '0')
    with open(file, "w", encoding='utf-8') as f:
        json.dump(data, f)


def json2dict(file):
    with open(file) as json_file:
        data = json.load(json_file)
    pass
    b642dict(data)
    return data 


def jsons2dict(file,name='ALLDATA'):
    with open(file) as json_file:
        data = json.load(json_file)
    pass
    if name == 'None':
        pass
    else:
        numpy2dict(data,file,name)
    return data 


def t1():
    dt = {}
    t = np.array([1,2,3], dtype=np.float32)
    t = np.arange(24).reshape(4, 3, 2).astype(np.float32)
    dt['t'] = t
    print(dt)
    dict2b64(dt) 
    print(dt)
    b642dict(dt)
    print(dt)


def test():
    t = {}
    t[1] = 1
    t[2] = np.array([0,1], dtype=np.float32)
    t[2] = np.arange(24).reshape(3, 4, 2).astype(np.float32)
    b = base64.b64encode(t[2])
    sb = str(b)
    data = base64.decodebytes(bytes(sb[2:-1], "utf8"))
    t1 = np.frombuffer(data, dtype=np.float32)
    print(t1)
    breakpoint()
    b64 = 'data:application/octet-stream;base64,'+str(b)[2:-1]
    t[2] = b64
    with open('t.json', "w", encoding='utf-8') as f:
        json.dump(t, f)
    exit()

# numpy数组转base64编码
    arr = np.arange(12).reshape(3, 4)
    bytesio = BytesIO()
    np.savetxt(bytesio, arr) # 只支持1维或者2维数组，numpy数组转化成字节流
    content = bytesio.getvalue()  # 获取string字符串表示
    print(content)
    b64_code = base64.b64encode(content)
     
# 从base64编码恢复numpy数组
    b64_decode = base64.b64decode(b64_code)
    arr = np.loadtxt(BytesIO(b64_decode))
    print(arr)
    breakpoint()


def main():
    # 数据
    dt = {}
    t = np.array([2,3], dtype=np.float32)
    t = np.arange(32).reshape(2, 2, 8).astype(np.int16)
    np.save('t.npy', t)
    dt['n'] = t
    dt['d'] = {1:4, 'a':'b'}
    dt['l'] = [1,2,3,'o']
    dt['n1'] = t+1
    import copy
    dt1 = copy.deepcopy(dt)

    # 将数据存为json
    dict2json(dt, 'dt.json')

    # 读取json数据
    dto = json2dict('dt.json')

    # 将数据存为json
    dict2jsons(dt1, 'dt1.json')

    # 读取json数据
    dto1 = jsons2dict('dt1.json', 'None')
    print(dto1)


if __name__ == "__main__":
    main()




import glob
import re
import os
import matplotlib.pyplot as plt
import numpy as np
from osgeo import gdal
import json
from tiff2png import createQGISMap
from tiff2png import createQGISMaps
import time
import cv2
import matplotlib.patheffects as path_effects
import time_htht as htt


def say(*args, **kw):
    print(*args, **kw, flush=True)
    return time.time()


def remove_white(figname, maxw=0.05, 
                 axis=0, outfile='', idy=''):
    im = plt.imread(figname).astype(np.float32)
    while im.shape[axis] <= 100:
        im = cv2.resize(im, [im.shape[1]*2, im.shape[0]*2])

    if str(type(idy)) == str(type('')):
        height, width = im.shape[0], im.shape[1]
        rgb = im[:,:,0]+im[:,:,1]+im[:,:,2]
        rgb[im[:,:,3] == 0] = 255 
        rgb = np.sum(rgb, 1-axis)
       
        w = im.shape[1-axis]*3*255
        w = np.max(rgb)
        rgb[0] = 0
        rgb[-1] = 0
        id = np.where(rgb < w)
        id = id[0]
        idb = id[1:] - id[0:-1]

        maxw = maxw * im.shape[axis]
        idb_gt_maxw = np.where(idb >  maxw)
        idb_gt_maxw = idb_gt_maxw[0]
        idy = rgb >= 0
        for i in idb_gt_maxw.tolist():
            idy[id[i]:id[i+1]] = False
            idy[id[i]:int(id[i]+maxw)] = True

    if axis == 0:
        im = im[idy, :, :]
    else:
        im = im[:, idy, :]
    
    if outfile == '':
        outfile = figname

    plt.imsave(outfile, np.ascontiguousarray(im))
    return idy


def clip_data(filepath, clipedpath, shpfile, filevalue=255):
    import rioxarray as rxr
    import geopandas as gpd
    from shapely.geometry import mapping
    ds1 = rxr.open_rasterio(filepath).squeeze()
    ds1 = ds1.rename({'x': 'longitude', 'y': 'latitude'})
    ds1.attrs["_FillValue"] = filevalue
    geodf = gpd.read_file(shpfile)
    try:
        ds1_new = ds1.rio.clip(
                geodf.geometry.apply(mapping),
                geodf.crs, drop=True)
        ds1_new.rio.to_raster(clipedpath)
    except Exception as e:
        print(e)


def writeTif_GLL(outputpath, data, lon, lat, 
                 miss=9999, datatype=gdal.GDT_Int16):
    # {{{
    wkt = 'GEOGCS["WGS 84",' \
                      'DATUM["WGS_1984",' \
                              'SPHEROID["WGS 84",6378137,298.257223563,' 'AUTHORITY["EPSG","7030"]],' \
                              'AUTHORITY["EPSG","6326"]],' \
                      'PRIMEM["Greenwich",0,AUTHORITY["EPSG","8901"]],' \
                      'UNIT["degree",0.0174532925199433,AUTHORITY["EPSG","9122"]],' \
                      'AXIS["Latitude",NORTH],' \
                      'AXIS["Longitude",EAST],' \
                      'AUTHORITY["EPSG","4326"]]'

    if len(data.shape) == 3:
        bands, height, width = data.shape
    else:
        bands, (height, width) = 1, data.shape
    driver = gdal.GetDriverByName('GTiff')
    dataset = driver.Create(outputpath, width, height, bands, datatype)
    if (dataset != None):
        dataset.SetProjection(wkt)  # 写入投影
        dataset.SetGeoTransform([lon[0], lon[1]-lon[0], 0.0, lat[0], 0.0, lat[1]-lat[0]])  # 写入仿射变换参数
    if bands == 1:
        dataset.GetRasterBand(1).WriteArray(data)
        dataset.GetRasterBand(1).SetNoDataValue(miss)
    else:
        for i in range(bands):
            dataset.GetRasterBand(i + 1).WriteArray(data[i])
            dataset.GetRasterBand(i + 1).SetNoDataValue(miss)
    dataset.FlushCache()
    del dataset
    driver = None
    # }}}


class fig(object):

    """Docstring for fig. """

    def __init__(self):
        """TODO: to be defined. """
        self.prodir = os.path.dirname(os.path.abspath(__file__))
        

    def png2qgs(self):
        # generate geotifffile
        # {{{
        # figname = '/home/leon/src/fy/amv.test.png'
        figname = self.pngname
        t = plt.imread(figname)
        lonlim = self.lonlim
        latlim = self.latlim
        a = t[:,:,3]
        longd = np.linspace(lonlim[0], lonlim[1], t.shape[1])
        latgd = np.linspace(latlim[0], latlim[1], t.shape[0])
        t = t * 255
        r = t[:,:,0]
        g = t[:,:,1]
        b = t[:,:,2]
        r[a == 0] = 999
        t[:,:,0] = r
        # t[2000:2100,2000:2100,:] = 1
        # t[2100:2200,2100:2200,:] = 255
        t = np.transpose(t[-1::-1,:,0:3], [2, 0, 1])
        tiffile = figname+'.tiff'
        writeTif_GLL(tiffile, t, longd, latgd, miss=999)
        # }}}

        # genrate qgs file
        # {{{
        try:
            pngfile = self.qgsfile
        except Exception as e:
            pngfile = self.pngname
            self.qgsfile = pngfile

        qgs_template = self.qgs_template
        chndict = {"title": self.ttl,
                   "date": self.rttl,
                   "satellite": self.instrument,
                   "resolution": self.resolution}

        if self.mpro:
            import multiprocessing
            ctx = multiprocessing.get_context('spawn')
            p = ctx.Process(target=tiff2png.createQGISMap, args=(tiffile, pngfile, qgs_template, chndict, self.colorbarfile, self.qgslevel))
            p.start()
            p.join()
        else:
            schndict = dict2str(chndict)
            cmd = 'python3 '+self.prodir+'/qgisfig/tiff2png_exe.py '+tiffile+' '+pngfile+' '+\
                    qgs_template+' "'+schndict+'" '+self.colorbarfile+' '+str(self.qgslevel)
            try:
                cmd = cmd + ' ' + self.et
            except Exception as e:
                print(e)
            say(cmd)
            os.system(cmd)

        self.qgsfile = pngfile
        # }}}


    def png2qgs_json(self):
        # generate geotifffile
        # {{{
        # figname = '/home/leon/src/fy/amv.test.png'
        print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
        tiffile = ''

        with open(self.qgs_json_file) as json_file:
            self.qgs_json = json.load(json_file)

        with open(self.png_json_file) as json_file:
            self.png_json = json.load(json_file)

        self.qgsfile = self.png_json['qgsfile']

        if 'tiff' in self.png_json:
            if os.path.exists(self.png_json['tiff']):
                tiffile = self.png_json['tiff']

        if tiffile == '':
            time0 = say('gen tiff')
            print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
            figname = self.pngfile
            t = plt.imread(figname)

            if 'cbpos' in self.png_json:
                cbpos = self.png_json['cbpos']
                xsize = t.shape[1]
                ysize = t.shape[0]
                ynd = int((1-cbpos[1])*ysize)
                yst = int((1-cbpos[1]-cbpos[3])*ysize)
                xst = int(cbpos[0]*xsize)
                xnd = int((cbpos[0]+cbpos[2])*xsize)
                cb = t[yst:ynd,xst:xnd,:]
                cbfile = self.pngfile+'.cb.tmp.png'
                plt.imsave(cbfile, cb)
                maxw0 = 0.01
                maxw1 = 0.01
                try:
                    maxw0 = float(self.png_json['maxw0'])
                except Exception as e:
                    pass
                try:
                    maxw1 = float(self.png_json['maxw1'])
                except Exception as e:
                    pass
                remove_white(cbfile, maxw=maxw0, axis=0)
                remove_white(cbfile, maxw=maxw1, axis=1)

                if 'cb_alpha' in self.png_json:
                    t1 = plt.imread(cbfile)
                    a1 = t1[:,:,3]

                    r1 = t1[:,:,0]
                    g1 = t1[:,:,1]
                    b1 = t1[:,:,2]
                    if 'cb_bg' in self.png_json:
                        r1[a1 == 0] = self.png_json['cb_bg'][0]
                        g1[a1 == 0] = self.png_json['cb_bg'][1]
                        b1[a1 == 0] = self.png_json['cb_bg'][2]
                    else:
                        r1[a1 == 0] = 0
                        g1[a1 == 0] = 0
                        b1[a1 == 0] = 0
                    a1[a1 == 0] = self.png_json['cb_alpha']

                    plt.imsave(cbfile, t1)

                if 'colorbar' in self.png_json:
                    pass
                else:
                    self.png_json['colorbar'] = cbfile
                    self.colorbarfile = cbfile
                    say('cmd:rm -rf '+cbfile)
                    with open(self.png_json_file, "w", encoding='utf-8') as f:
                        json.dump(self.png_json, f)
                    pass

            if 'axpos' in self.png_json:
                axpos = self.png_json['axpos']
                xsize = t.shape[1]
                ysize = t.shape[0]
                ynd = int((1-axpos[1])*ysize)
                yst = int((1-axpos[1]-axpos[3])*ysize)
                xst = int(axpos[0]*xsize)
                xnd = int((axpos[0]+axpos[2])*xsize)
                t = t[yst:ynd,xst:xnd,:]

            print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
            lonlim = self.png_json['lonlim']
            latlim = self.png_json['latlim']
            longd = np.linspace(lonlim[0], lonlim[1], t.shape[1])
            latgd = np.linspace(latlim[0], latlim[1], t.shape[0])
            t = t * 255
            r = t[:,:,0]
            g = t[:,:,1]
            b = t[:,:,2]
            try:
                a = t[:,:,3]
                r[a == 0] = 999
            except Exception as e:
                print(e)
            t[:,:,0] = r
            # t[2000:2100,2000:2100,:] = 1
            # t[2100:2200,2100:2200,:] = 255
            t = np.transpose(t[-1::-1,:,0:3], [2, 0, 1])
            try:
                tiffile = self.png_json['tiff']
            except Exception as e:
                tiffile = figname+'.'+str(time.time())+'.tiff'
            if 'tiffdatatype' in self.png_json:
                if self.png_json['tiffdatatype'] == 'uint8':
                    t1 = np.ones([t.shape[0]+1, t.shape[1], t.shape[2]])
                    t1[0:3, :, :] = t
                    t1[3, :, :] = 255
                    r = t1[0, :, :]
                    g = t1[1, :, :]
                    b = t1[2, :, :]
                    a = t1[3, :, :]
                    a[(r > 255) | (g > 255) | (g > 255)] = 0
                    t1[t1 > 255] = 255
                    t1[t1 < 0] = 0
                    writeTif_GLL(tiffile, t1, longd, latgd, 
                                 miss=999, datatype=gdal.GDT_Byte)
            else:
                writeTif_GLL(tiffile, t, longd, latgd, miss=999)
            say('gen tiff finished', time.time()-time0)
            print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))

            if 'outpng' in self.png_json:
                t1 = np.ones([t.shape[1], t.shape[2], 4])
                t1[:, :, 0:3] = t.transpose(1,2,0)/255
                r = t1[:, :, 0] 
                g = t1[:, :, 1]
                b = t1[:, :, 2]
                a = t1[:, :, 3]
                id = (r > 1) | (g > 1) | (b > 1)
                r[id] = 1
                g[id] = 1
                b[id] = 1
                a[id] = 0
                plt.imsave(self.png_json['outpng'], t1)

        tiffile1 = tiffile
        try:
            if self.png_json['removeout'] == 'yes':
                t0 = say('clip tiff')
                print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
                try:
                    shpfile = self.png_json['bounds']
                except Exception as e:
                    shpfile = re.sub(r'[^\/\\]+$', 'bounds.shp', 
                            self.qgs_template)
                tiffile1 = tiffile + '.cliptiff.'+str(time.time())+'.tiff'
                clip_data(tiffile, tiffile1, shpfile, filevalue=999)
                try:
                    if 'keeptiff' in self.png_json:
                        pass
                    else:
                        safe_remove(tiffile)
                except Exception as e:
                    print(e)
                say('clip tiff finished', time.time()-t0)
                print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
        except Exception as e:
            print(e)
            pass
        # }}}

        # genrate qgs file
        # {{{
        print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
        chndict = self.qgs_json['label']
        for k in chndict.keys():
            try:
                chndict[k] = self.png_json[k]
            except Exception as e:
                pass
        self.qgslevel = int(self.qgs_json['level'])
        try:
            self.qgslevel = self.png_json['qgslevel']
        except Exception as e:
            pass

        deletepng = True
        try:
            if self.png_json['deletepng'] == 'no':
                deletepng = True
        except Exception as e:
            pass

        et = ''
        try:
            et = self.png_json['extent']
        except Exception as e:
            pass

        self.colorbarfile = ''
        try:
            self.colorbarfile = self.png_json['colorbar']
        except Exception as e:
            pass

        mapid='Map1'
        try:
            mapid = self.qgs_json['mapid']
        except Exception as e:
            pass

        t0 = say('start convert qgis')
        removetiff = True
        if 'keeptiff' in self.png_json:
            removetiff = False
        else:
            pass

        try:
            for k in chndict.keys():
                chndict[k] = re.sub(r'号台风热带低压', '号热带低压', chndict[k])
                chndict[k] = re.sub(r'号台风"热带低压"', '号热带低压', chndict[k])
        except Exception as e:
            print(e)
        print(self.colorbarfile)

        if 'outcolorbar' in self.png_json:
            shutil.copy2(self.colorbarfile, self.png_json['outcolorbar'])

        if 'noqgs' in self.png_json:
            say('end convert tiff', time.time()-t0)
            return

        try:
            createQGISMap(tiffile1, self.qgsfile, self.qgs_template, 
                    chndict, self.colorbarfile, level=self.qgslevel, 
                    et=et, mapid=mapid, removetiff=removetiff)
        except Exception as e:
            print(e)
        
        say('end convert qgis', time.time()-t0)
        print(htt.time2str(time.time(), 'yyyy:mm:dd HH:MM:SS'))
        # }}}

    def pngs2qgs_json(self):
        # generate geotifffile
        # {{{
        # figname = '/home/leon/src/fy/amv.test.png'

        with open(self.qgs_json_file) as json_file:
            self.qgs_json = json.load(json_file)

        with open(self.png_json_file) as json_file:
            self.png_json = json.load(json_file)

        fignames = self.pngfile
        tiffs = []
        for figname in re.findall(r'[^\,]+', fignames):
            self.qgsfile = self.png_json['qgsfile']

            time0 = say('gen tiff')
            t = plt.imread(figname)
            lonlim = self.png_json['lonlim']
            latlim = self.png_json['latlim']
            a = t[:,:,3]
            longd = np.linspace(lonlim[0], lonlim[1], t.shape[1])
            latgd = np.linspace(latlim[0], latlim[1], t.shape[0])
            t = t * 255
            r = t[:,:,0]
            g = t[:,:,1]
            b = t[:,:,2]
            r[a == 0] = 999
            t[:,:,0] = r
            # t[2000:2100,2000:2100,:] = 1
            # t[2100:2200,2100:2200,:] = 255
            t = np.transpose(t[-1::-1,:,0:3], [2, 0, 1])
            tiffile = figname+'.tiff'
            writeTif_GLL(tiffile, t, longd, latgd, miss=999)
            say('gen tiff finished', time.time()-time0)

            tiffile1 = tiffile
            try:
                if self.png_json['removeout'] == 'yes':
                    t0 = say('clip tiff')
                    try:
                        shpfile = self.png_json['bounds']
                    except Exception as e:
                        shpfile = re.sub('[^\/]+$', 'bounds.shp', 
                                self.qgs_template)
                    tiffile1 = tiffile + '.'+str(time.time())+'.tiff'
                    clip_data(tiffile, tiffile1, shpfile, filevalue=999)
                    try:
                        if 'keeptiff' in self.png_json:
                            pass
                        else:
                            safe_remove(tiffile)
                    except Exception as e:
                        pass
                    say('clip tiff finished', time.time()-t0)
            except Exception as e:
                pass
            tiffs.append(tiffile1)
            # }}}

        # genrate qgs file
        # {{{
        chndict = self.qgs_json['label']
        for k in chndict.keys():
            try:
                chndict[k] = self.png_json[k]
            except Exception as e:
                pass
        self.qgslevel = int(self.qgs_json['level'])
        try:
            self.qgslevel = self.png_json['qgslevel']
        except Exception as e:
            self.qgslevel = str(self.qgslevel-2)+','+str(self.qgslevel)

        deletepng = True
        try:
            if self.png_json['deletepng'] == 'no':
                deletepng = True
        except Exception as e:
            pass

        et = ''
        try:
            et = self.png_json['extent']
        except Exception as e:
            pass

        self.colorbarfile = ''
        try:
            self.colorbarfile = self.png_json['colorbar']
        except Exception as e:
            pass
        mapid='Map1'
        try:
            mapid = self.qgs_json['mapid']
        except Exception as e:
            pass

        t0 = say('start convert qgis')
        removetiff = True
        if 'keeptiff' in self.png_json:
            removetiff = False
        else:
            pass

        for k in chndict.keys():
            chndict[k] = re.sub(r'号台风热带低压', '号热带低压', chndict[k])
            chndict[k] = re.sub(r'号台风"热带低压"', '号热带低压', chndict[k])
        createQGISMaps(tiffs, self.qgsfile, self.qgs_template, 
                chndict, self.colorbarfile, levels=self.qgslevel, 
                et=et, mapid=mapid, removetiff=removetiff)
        say('end convert qgis', time.time()-t0)
        # }}}


def convert_png2qgs(infile, instrument, resolution, qgs_template,
                    colorbarfile, ttl, rttl, lonlim, latlim, et,
                    figname, qgslevel, qgsfile):
    sinfile = re.sub('.*\/', '', infile)
    a = fig()
    a.instrument = instrument
    a.resolution = resolution
    a.qgs_template = qgs_template
    a.pngname = infile
    a.qgsfile = qgsfile
    a.colorbarfile = colorbarfile
    a.ttl = ttl
    a.rttl = rttl
    a.lonlim = lonlim
    a.latlim = latlim
    a.et = et
    a.mpro = False
    a.qgslevel = qgslevel
    a.png2qgs()


def t1_convert_png2qgs():
    for infile in glob.glob('/home/leon/Documents/台风大风/*png'):
        sinfile = re.sub('.*\/', '', infile)
        tmp = re.search('^([^_]+)_(.*)\.png$', sinfile) 
        instrument = 'FY3D/MWRI'
        resolution = '15km'
        qgs_template = '/home/leon/src/qgisfig/glob/template.qgs'
        qgsfile = '/home/leon/src/fy/tmp/wind_'+sinfile+'.q.png'
        colorbarfile = './bai1.png'
        colorbarfile1 = '/home/leon/Documents/台风大风/colorbar/'+\
                tmp.group(1)+'.png'
        if os.path.exists(colorbarfile1):
            colorbarfile = colorbarfile1
        ttl = '台风'+tmp.group(1)
        rttl = '2021-07-21 11:00:00(北京时)'
        t2 = re.findall('[^_]+', tmp.group(2))
        lonlim = [float(t2[0]), float(t2[2])]
        latlim = [float(t2[1]), float(t2[3])]
        dlon = (lonlim[1]-lonlim[0])/10
        dlat = (latlim[1]-latlim[0])/10
        print(dlon)
        print(dlat)
        et = str(lonlim[0]+dlon)+'_'+str(latlim[0]+dlat)+'_'\
                +str(lonlim[1]-dlon)+'_'+str(latlim[1]-dlat)
        figname = tmp.group(1)
        qgslevel = 2
        convert_png2qgs(infile, instrument, resolution, qgs_template,
                colorbarfile, ttl, rttl, lonlim, latlim, et, figname, qgslevel, qgsfile)


def test_convert_png2qgs():
    for infile in glob.glob('/home/leon/Documents/台风大风/*png'):
        sinfile = re.sub('.*\/', '', infile)
        tmp = re.search('^([^_]+)_(.*)\.png$', sinfile) 
        a = fig()
        a.instrument = 'FY3D/MWRI'
        a.resolution = '15km'
        a.qgs_template = '/home/leon/src/qgisfig/glob/template.qgs'
        a.pngname = infile
        a.qgsfile = '/home/leon/src/fy/tmp/wind_'+sinfile+'.q.png'
        a.colorbarfile = './bai1.png'
        colorbarfile = '/home/leon/Documents/台风大风/colorbar/'+\
                tmp.group(1)+'.png'
        if os.path.exists(colorbarfile):
            a.colorbarfile = colorbarfile
        a.ttl = '台风'+tmp.group(1)
        a.rttl = '2021-07-21 11:00:00(北京时)'
        t2 = re.findall('[^_]+', tmp.group(2))
        a.lonlim = [float(t2[0]), float(t2[2])]
        a.latlim = [float(t2[1]), float(t2[3])]
        dlon = (a.lonlim[1]-a.lonlim[0])/10
        dlat = (a.latlim[1]-a.latlim[0])/10
        print(dlon)
        print(dlat)
        a.et = str(a.lonlim[0]+dlon)+'_'+str(a.latlim[0]+dlat)+'_'\
                +str(a.lonlim[1]-dlon)+'_'+str(a.latlim[1]-dlat)
        a.figname = tmp.group(1)
        print(a.et)
        print(a.lonlim)
        print(a.latlim)
        print(a.lonlim)
        a.mpro = False
        a.qgslevel = 2
        a.png2qgs()


def test_exe():
    fig = plt.figure(figsize=(9,10))
    axpos = (0.1,0.4,0.8,0.5)
    ax = fig.add_axes(axpos)
    it = ax.imshow(np.array([[1,2],[3,4]]), aspect='auto')
    ax1 = fig.add_axes([0.15,0.1,0.6,0.02])
    cb = plt.colorbar(it, cax=ax1, orientation='horizontal')
    ym = ax1.get_ylim()
    xm = ax1.get_xlim()
    dy = ym[1]-ym[0]
    ax1.text(xm[0], ym[0]+dy/2, 'xxxx  ',
            horizontalalignment='right', verticalalignment='center')
    ax1.text(xm[1], ym[0]+dy/2, '  xxxx',
            horizontalalignment='left', verticalalignment='center')
    ax1.set_xticklabels(ax1.get_xticks())
    fe = [path_effects.Stroke(linewidth=3, foreground='black'), path_effects.Normal()]
    for tax in plt.gcf().axes:  
        # {{{
        ts = tax.get_children()
        ts.extend(tax.get_xticklabels())
        for t in ts:
            try:
                if re.search('Text', str(type(t))):
                    t.set_path_effects(fe)
                    t.set_color('w')
            except Exception as e:
                pass  # }}}

    fig.savefig('5.png', transparent=True, dpi=200)
    pngdict = {'title': 'xxxxxxxxxxx', 
               'date': '0000-00-00',
               'lonlim': [0,180], 
               'latlim': [-20, 80], 
               'qgsfile': '5.qgs.png',
               'satellite': 'xxx/xx',
               'axpos': axpos,
               'cbpos': [0,0,1,0.2],
               'qgslevel': 12,
               'qgsfile': '5.qgsc.png',
               'removeout': 'yes',
               'resolution': '??km'}

    with open('5.png.json', "w", encoding='utf-8') as f:
        json.dump(pngdict, f)

    cmd = 'python3 exe_png2qgis.py --png=5.png --qgs=/.out/home/leon/src/qgis/new/weather-ll/template-regc.qgs'
    print(cmd)
    os.system(cmd)
    

if __name__ == "__main__":
    # t1_convert_png2qgs()
    test_exe()
    
    exit()
    clip_data('', clipedpath, shpfile, filevalue=255)




from setuptools import setup, find_packages

setup(
    name="toothedsword",
    version="0.2.1",
    description="nothing",
    packages=find_packages(),
    python_requires=">=3.6",
    include_package_data=True,

    package_data={
        "toothedsword": ["base/*.json", 
                         "tctb/*.json", 
                         "ningxia/*.json"],
                 },
    install_requires=[
        "numpy>=1.12.0",
        "pyproj>=1.0.6",
        "matplotlib>=1.0.0",
        "opencv-python>=3.0.0",
        "rioxarray>=0.0.1",
        "geopandas>=0.1.0",
        "GDAL>=1.5.0",
    ],  
       
       
)




import sys
import json
import os
import re
import glob
# from mayavi import mlab
# import time_htht as htt
from toothedsword import time_htht as htt
import numpy as np
# import pandas as pd
import copy
import matplotlib as mpl
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.widgets import Cursor, Button, CheckButtons
# from osgeo import gdal
# from skimage import morphology
import time
from matplotlib import colors
from matplotlib import cm
from matplotlib import path
import matplotlib
import matplotlib.patheffects as path_effects
import xml.etree.ElementTree as ET
from pyproj import Transformer
from .common import safe_remove


from matplotlib.font_manager import FontProperties
yh_font = FontProperties(fname='/usr/share/fonts/msyh.ttc')
kai_font = FontProperties(fname='/usr/share/fonts/simkai.ttf')
hei_font = FontProperties(fname='/usr/share/fonts/simhei.ttf')


def get_layout_extent(project_file):

# 读取 QGIS 项目文件
    # project_file = "./template.qgs"
    tree = ET.parse(project_file)
    root = tree.getroot()

# 查找地图项（LayoutItemMap）
    for item in root.findall(".//LayoutItem[@type='65639']"):
        extent = item.find("Extent")
        if extent is not None:
            # 提取投影坐标
            xmin = float(extent.get("xmin"))
            xmax = float(extent.get("xmax"))
            ymin = float(extent.get("ymin"))
            ymax = float(extent.get("ymax"))

            # 查找 CRS 信息
            crs = item.find(".//crs/spatialrefsys/authid")
            if crs is not None:
                source_crs = crs.text  # 例如 "EPSG:4499"
                target_crs = "EPSG:4326"  # 目标坐标系（WGS84 经纬度）

                # 创建坐标转换器
                transformer = Transformer.from_crs(source_crs, target_crs, always_xy=True)

                # 将投影坐标转换为经纬度坐标
                lon_min, lat_min = transformer.transform(xmin, ymin)
                lon_max, lat_max = transformer.transform(xmax, ymax)

                # 打印结果
                print(f"地图项范围 (投影坐标):")
                print(f"  xmin: {xmin}, ymin: {ymin}")
                print(f"  xmax: {xmax}, ymax: {ymax}")
                print(f"地图项范围 (经纬度坐标):")
                print(f"  经度范围: {lon_min} 到 {lon_max}")
                print(f"  纬度范围: {lat_min} 到 {lat_max}")
                return float(lon_min), float(lon_max), \
                        float(lat_min), float(lat_max)
            else:
                print("未找到 CRS 信息。")
        else:
            print("未找到 <Extent> 元素。")

def get_qgis_extent(template_file):
# 读取 QGIS 模板文件
# template_file = "template.qgs"
    tree = ET.parse(template_file)
    root = tree.getroot()

# 查找 DefaultViewExtent 元素
    default_view_extent = root.find(".//DefaultViewExtent")

# 提取 xmax, xmin, ymin, ymax
    if default_view_extent is not None:
        xmax = default_view_extent.get("xmax")
        xmin = default_view_extent.get("xmin")
        ymin = default_view_extent.get("ymin")
        ymax = default_view_extent.get("ymax")

        # 打印结果
        print(f"xmax: {xmax}")
        print(f"xmin: {xmin}")
        print(f"ymin: {ymin}")
        print(f"ymax: {ymax}")
        return float(xmin), float(xmax),\
                float(ymin), float(ymax)
    else:
        print("DefaultViewExtent not found in the template file.")

def remove_white(figname, maxw=0.05, axis=0, outfile='', idy=''):
    im = plt.imread(figname).astype(np.float32)
    if str(type(idy)) == str(type('')):
        height, width = im.shape[0], im.shape[1]
        rgb = im[:,:,0]+im[:,:,1]+im[:,:,2]
        rgb[im[:,:,3] == 0] = 255 
        rgb = np.sum(rgb, 1-axis)
       
        w = im.shape[1-axis]*3*255
        w = np.max(rgb)
        rgb[0] = 0
        rgb[-1] = 0
        id = np.where(rgb < w)
        id = id[0]
        idb = id[1:] - id[0:-1]

        maxw = maxw * im.shape[axis]
        idb_gt_maxw = np.where(idb >  maxw)
        idb_gt_maxw = idb_gt_maxw[0]
        idy = rgb >= 0
        for i in idb_gt_maxw.tolist():
            idy[id[i]:id[i+1]] = False
            idy[id[i]:int(id[i]+maxw)] = True

    if axis == 0:
        im = im[idy, :, :]
    else:
        im = im[:, idy, :]
    
    if outfile == '':
        outfile = figname

    plt.imsave(outfile, np.ascontiguousarray(im))
    return idy


def figure(*args, add_axes=False, **kw):
    fig = plt.figure(*args, **kw, FigureClass=FIG)
    fig.plt = plt
    fig.yh_font = yh_font
    fig.kai_font = kai_font
    fig.hei_font = hei_font
    if add_axes:
        ax = fig.add_axes_auto()
        fig.ax = ax
    return fig


class FIG(plt.Figure):
    """Docstring for . """
    def init(self):
        try:
            dpi = self.dpi
        except Exception as e:
            self.dpi = 400

        try:
            t = self.fig_varunit
        except Exception as e:
            self.fig_varunit = ''

        try:
            t = self.fig_varname
        except Exception as e:
            self.fig_varname = ''

        self.qgs_template = '/home/leon/src/qgisfig/new/achn/template.qgs'
        self.qgs_template = '/home/leon/src/qgisfig/glob/template.qgs'
        self.qgs_template = '/home/leon/src/qgis-050/dynamic-range/template.qgs'
        self.fontproperties = kai_font
        self.fontcolor = '#000000'
        self.pngfile = str(time.time())+'.png'
        self.png_json_file = self.pngfile + '.json'
        self.outfile = self.pngfile+'.png'
        self.time = time.time()
        self.ttl = 'xxxxx'
        self.sat = 'xxx'
        self.axpos = self.ax.get_position().bounds
        try:
            t = self.cb_axpos_out
        except Exception as e:
            self.cb_axpos_out = [0,0,1,0.2]
        self.res = 'xxx'
        self.prodir = os.path.dirname(os.path.abspath(__file__))
        self.qgs_template_json = self.qgs_template + '.json'
        self.update_png_json = {}
        self.say = print

    def fontzoom(self, scale):
        for text_obj in self.findobj(match=type(plt.Text)):
            try:
                text_obj.set_fontsize(text_obj.get_fontsize() * scale)
            except Exception as e:
                pass

    def remove_close_text(self, mindis, text_objs, rt='CC'):
        xlim = self.ax.get_xlim()
        ylim = self.ax.get_ylim()
        def dis(x1,y1,x0,y0,xlim,ylim):
            x1 = (x1 - xlim[0])/(xlim[1]-xlim[0])
            y1 = (y1 - ylim[0])/(ylim[1]-ylim[0])
            x0 = (x0 - xlim[0])/(xlim[1]-xlim[0])
            y0 = (y0 - ylim[0])/(ylim[1]-ylim[0])
            return np.sqrt((x1-x0)**2+(y1-y0)**2)

        for text_obj in text_objs:
            try:
                s0 = text_obj.get_text()
                x0, y0 = text_obj.get_position()
                if s0 == ' ':
                    continue
                
                for text_obj1 in text_objs:
                    try:
                        s1 = text_obj1.get_text()
                        x1, y1 = text_obj1.get_position()
                        d = dis(x1,y1,x0,y0,xlim,ylim)
                        if d > 0 and d < mindis and not(s1 == ' '):
                            text_obj1.set_text(' ')
                        
                    except Exception as e1:
                        print('---------------')
                        print(e1)
            except Exception as e:
                print('---------------')
                print(e)

    def add_axes_auto(self, type='main'):
        if type == 'main':
            ax = self.add_axes([0.15, 0.15, 0.7, 0.7])
            self.ax = ax
        if re.search('cb', type):
            if re.search('h', type):
                ax = self.add_axes([0.15, 0.15, 0.7, 0.02])
            else:
                ax = self.add_axes([0.87, 0.15, 0.02, 0.7])
            self.cax = ax
        return ax

    def set_timetick(self, xy='x', ss='yyyymmddHHMMSS'):
        ax = self.ax
        xticks = ax.get_xticks()
        xticklabels = []
        for x in xticks:
            xticklabels.append(htt.time2str(x, ss))
        ax.set_xticks(xticks)
        ax.set_xticklabels(xticklabels)

    def add_space(self, dr, wd):
        ax = self.ax
        current_position = ax.get_position()
        current_position_list = [current_position.x0, current_position.y0, current_position.width, current_position.height]
        
        try:
            cax = self.cax
            cposition = cax.get_position()
            cposition_list = [cposition.x0, cposition.y0, cposition.width, cposition.height]
        except Exception as e:
            print(e)

        if dr == 'top':
            current_position_list[3] -= wd
            cposition_list[3] -= wd

        if dr == 'right':
            current_position_list[2] -= wd
            cposition_list[0] -= wd

        if dr == 'bottom':
            current_position_list[3] -= wd
            current_position_list[1] += wd
            cposition_list[3] -= wd
            cposition_list[1] += wd

        if dr == 'left':
            current_position_list[2] -= wd
            current_position_list[0] += wd

        ax.set_position(current_position_list)
        try:
            cax.set_position(cposition_list)
        except Exception as e:
            print(e)

    def save(self, *args, maxw=[1,1], **kw):
        self.savefig(*args, **kw)
        fname = args[0]
        remove_white(fname, maxw=maxw[0], axis=0)
        remove_white(fname, maxw=maxw[1], axis=1)

    def set_axes_thick(self, thick):
        # for ax in self.axes:
        for ax in self.get_axes():
            # 获取当前Axes对象的边框
            spines = ax.spines

            # 设置边框线宽度为3
            for spine in spines.values():
                spine.set_linewidth(thick)

    def add_colorbar(self):
        self.ax_cb = self.fig.add_axes(self.cb_axpos_in)
        cb = plt.colorbar(self.it, cax=self.ax_cb, 
                orientation='horizontal')

    def addcolorbar(self, im, cax='', 
                 orientation='horizontal', 
                 var='xxx', unit='xxx'):
        
        if str(type(cax)) == str(type('')):
            if orientation=='horizontal':
                cax = self.add_axes([0.2, 0.1, 0.6, 0.02])
                self.cb_axpos_out = [0,0,1,0.2]
            if orientation=='vertical':
                cax = self.add_axes([0.8, 0.1, 0.02, 0.8])
                self.cb_axpos_out = [0.7,0,0.3,1]
            self.ax_cb = cax
        

        plt.colorbar(im, cax=cax, orientation=orientation)
        if var == 'xxx':
            pass
        else:
            self.fig_varname = var

        if unit == 'xxx':
            pass
        else:
            self.fig_varunit = unit

    def copy_attr_from_parent(self):
        pass

    def add_ll_unit_ax(self):
        # {{{
        ax = self.ax
        ts = []
        for t in ax.get_xticklabels():
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'$^{o}$W'
            else:
                t1 = t.get_text()+'$^{o}$E'
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)

        ts = []
        for t in ax.get_yticklabels():
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'$^{o}$S'
            else:
                t1 = t.get_text()+'$^{o}$N'
            ts.append(t1)
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels(ts)
        # }}}

    def add_tick_space(self, xy):
        # {{{
        ax = self.ax
        ts = []
        it = 1
        for t in ax.get_xticklabels():
            it += 1
            t1 = t.get_text()+''
            if xy[0] > 0:
                if it % xy[0] == 1:
                    pass
                else:
                    t1 = ' '
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)

        ts = []
        it = 0
        for t in ax.get_yticklabels():
            it += 1
            t1 = t.get_text()+''
            if xy[1] > 0:
                if it % xy[1] == 1:
                    t1 = ' '
            ts.append(t1)
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels(ts)
        # }}}

    def add_ll_unit_ax1(self):
        # {{{
        ax = self.ax
        ts = []
        it = 1
        for t in ax.get_xticklabels():
            it += 1
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'W'
            else:
                t1 = t.get_text()+'E'
            if it % 3 == 1:
                pass
            else:
                t1 = ' '
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)

        ts = []
        it = 0
        for t in ax.get_yticklabels():
            it += 1
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'S'
            else:
                t1 = t.get_text()+'N'
            if it % 2 == 1:
                t1 = ' '
            ts.append(t1)
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels(ts)
        # }}}

    def add_ll_unit_ax2(self):
        # {{{
        ax = self.ax
        ts = []
        it = 1
        for t in ax.get_xticklabels():
            it += 1
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'W'
            else:
                t1 = t.get_text()+'E'
            if it % 2 == 1:
                pass
            else:
                t1 = ' '
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)

        ts = []
        it = 0
        for t in ax.get_yticklabels():
            it += 1
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'S'
            else:
                t1 = t.get_text()+'N'
            if it % 2 == 1:
                t1 = ' '
            ts.append(t1)
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels(ts)
        # }}}


    def add_ll_unit_ax0(self):
        # {{{
        ax = self.ax
        ts = []
        it = 1
        for t in ax.get_xticklabels():
            it += 1
            t1 = t.get_text()+''
            if it % 2 == 1:
                pass
            else:
                t1 = ' '
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)

        ts = []
        it = 0
        for t in ax.get_yticklabels():
            it += 1
            t1 = t.get_text()+''
            if it % 2 == 1:
                t1 = ' '
            ts.append(t1)
        ax.set_yticks(ax.get_yticks())
        ax.set_yticklabels(ts)
        # }}}

    def set_fmt_ax(self):
        # {{{ 
        ax = self.ax
        ts = []
        i = 0
        for t in ax.get_xticklabels():
            i += 1
            t1 = t.get_text()+''
            if float(t.get_text()) == 0:
                pass
            elif re.search(r'-', t.get_text()):
                t1 = re.sub(r'-','',t.get_text())+'$^{o}$S'
            else:
                t1 = t.get_text()+'$^{o}$N'
            t1 = re.sub(r'(\d.\d)\d*', r'\1', t1)
            if i % 2 == 1:
                t1 = ' '
            ts.append(t1)
        ax.set_xticks(ax.get_xticks())
        ax.set_xticklabels(ts)
        # }}}

    def save_png(self):
        # {{{
        pngfile = self.pngfile
        fig = self
        ax = self.ax
        ax_cb = self.ax_cb
        ym = ax_cb.get_ylim()
        xm = ax_cb.get_xlim()
        dy = ym[1]-ym[0]
        dx = xm[1]-xm[0]
        pos = ax_cb.get_position()
      
        if pos.width > pos.height:
            if not(self.fig_varname == ''):
                ax_cb.text(xm[0], ym[0]+dy/2, 
                           self.fig_varname+'  ', 
                           fontproperties=self.fontproperties,
                           horizontalalignment='right', 
                           verticalalignment='center')
            if not(self.fig_varunit == ''):
                ax_cb.text(xm[1], ym[0]+dy/2, 
                           '  '+self.fig_varunit, 
                           fontproperties=yh_font,
                           horizontalalignment='left', 
                           verticalalignment='center')
        else:
            if not(self.fig_varname == ''):
                ax_cb.text(xm[0]+dx/2, ym[0]-dy/100, 
                           self.fig_varname, 
                           fontproperties=self.fontproperties,
                           horizontalalignment='center', 
                           verticalalignment='top')
            if not(self.fig_varunit == ''):
                ax_cb.text(xm[0]+dx/2, ym[1]+dy/100,
                           self.fig_varunit, 
                           fontproperties=self.fontproperties,
                           horizontalalignment='center', 
                           verticalalignment='bottom')

        if self.fontcolor == '#ffffff':
            fe = [path_effects.Stroke(linewidth=3, foreground='black'), path_effects.Normal()]
            for tax in [ax, ax_cb]: 
                # {{{
                ts = tax.get_children()
                ts.extend(tax.get_xticklabels())
                for t in ts:
                    try:
                        if re.search(r'Text', str(type(t))):
                            t.set_path_effects(fe)
                            t.set_color('w')
                    except Exception as e:
                        print(e)
                        # }}}

        outdir = re.sub(r'[^\/]+$', '', self.outfile)
        try:
            os.makedirs(outdir, exist_ok=True)
        except Exception as e:
            print(e)
        fig.savefig(pngfile, transparent=True, dpi=self.dpi)
        # }}}

    def gen_png_dict(self):
        # {{{
        fig = self
        ax = self.ax
        ax_cb = self.ax_cb
        self.textspan0 = ''
        self.textspan1 = ''

        if self.fontcolor == '#ffffff':
            self.textspan0 = '<p style="color:'+self.fontcolor+\
                    ';-webkit-text-stroke: 4px black">'
            self.textspan1 = '</p>'

        stime = htt.time2str(self.time, 'yyyymmddHHMMSS')
        outfile = self.outfile

        outdir = re.sub(r'[^\/]+$', '', outfile)
        try:
            os.makedirs(outdir, exist_ok=True)
        except Exception as e:
            print(e)

        lonlim = ax.get_xlim()
        latlim = ax.get_ylim()

        pngdict = {'title': self.textspan0+self.ttl+self.textspan1, 
                   'date': self.textspan0+htt.time2str(self.time+8*3600, 'yyyy-mm-dd HH:MM')+'(北京时间)'+self.textspan1,
                   'lonlim': [lonlim[0], lonlim[1]], 
                   'latlim': [latlim[0], latlim[1]], 
                   'qgsfile': outfile,
                   'satellite': self.textspan0+'卫星: '+self.sat+self.textspan1,
                   'axpos': self.axpos,
                   'cbpos': self.cb_axpos_out,
                   "tuli": self.textspan0+"图 例"+self.textspan1,
                   "guojie": self.textspan0+"国界"+self.textspan1,
                   "shengjie": self.textspan0+"省界"+self.textspan1,
                   "haiyang": self.textspan0+"海洋"+self.textspan1,
                   "ludi": self.textspan0+"陆地"+self.textspan1,
                   'resolution': self.textspan0+'分辨率: '+self.res+self.textspan1}
        try:
            pngdict['date'] = self.textspan0+self.rttl+self.textspan1
        except Exception as e:
            print(e)
        try:
            for k in self.pngdict_add.keys():
                pngdict[k] = self.pngdict_add[k]
        except Exception as e:
            pass
        for k in pngdict.keys():
            try:
                if re.search(r'^###', pngdict[k]):
                    pngdict[k] = self.textspan0+\
                            re.sub(r'^###', self.textspan0, 
                                    pngdict[k])+self.textspan1
            except Exception as e:
                print(e)
        self.pngdict = pngdict
        # }}}

    def save_png_json(self):
        # {{{
        pngfile = self.pngfile
        png_json_file = pngfile+'.tmp.json'

        with open(png_json_file, "w", encoding='utf-8') as f:
            json.dump(self.pngdict, f)
        self.png_json_file = png_json_file
        # }}}

    def gen_qgis(self, run=True):
        # {{{
        pngfile = self.pngfile

        cmd = 'python3 '+self.prodir+'/exe_png2qgis.py --png='+pngfile+' --qgs='+self.qgs_template+' --png_json='+self.png_json_file
        if self.qgs_template_json == '':
            pass
        else:
            cmd += ' --qgs_json='+self.qgs_template_json
        print(cmd)
        if run:
            os.system(cmd)
        return cmd
        pass
        # }}}

    def save2qgis(self, outfile='', qgs_template='', 
                  qgs_template_json='',info={}, run=True):
        """
        fig.save2qgis('1.png', 
              '/home/leon/src/qgis/new/achn/template_test.qgs',
              info={'varname':'亮温', 'varunit':'K',
                    'title':'测试图片', 'date':a.stime, 
                    'satellite':'卫星/载荷:FY4B/AGRI',
                    'removeout':'yes', 'qgslevel':12,
                    'resolution':'分辨率:4公里'})
        """
        self.info = info
        try:
            t = self.ax
        except Exception as e:
            self.ax = self.axes[0]

        if 'gextent' in info:
            if info['gextent'] == 'qgs':
                # xmin, xmax, ymin, ymax =\
                #         get_qgis_extent(qgs_template)
                xmin, xmax, ymin, ymax =\
                        get_layout_extent(qgs_template)
                if xmin > -180 and xmin < 180 and\
                   xmax > -180 and xmax < 180 and\
                   ymin > -90 and ymin < 90 and\
                   ymax > -90 and ymax < 90:
                    self.ax.set_xlim([xmin, xmax])
                    self.ax.set_ylim([ymin, ymax])
        
        try:
            for i in range(0, len(self.clabel)):
                try:
                    self.clabel[i].remove()
                except Exception as e:
                    pass
        except Exception as e:
            print(e)

        try:
            self.clabel = self.plt.clabel(self.ct, inline=False,
                                 inline_spacing=1, fontsize=7)
            pass
        except Exception as e:
            print(e)

        try:
            self.ct.set_linewidths(self.ct_lw)
        except Exception as e:
            print(e)

        if 'varname' in info:
            self.fig_varname = info['varname']

        if 'varunit' in info:
            self.fig_varunit = info['varunit']

        try:
            t = self.ax_cb
        except Exception as e:
            try:
                self.ax_cb = self.axes[1]
            except Exception as e:
                pass

        self.init()
        if outfile=='':
            pass
        else:
            self.outfile = outfile
            self.pngfile = self.outfile + '.tmp.png'
       
        if qgs_template=='':
            pass
        else:
            self.qgs_template = qgs_template
 
        if qgs_template_json=='':
            self.qgs_template_json = self.qgs_template+'.json'
        else:
            self.qgs_template_json = qgs_template_json

        if os.path.exists(self.qgs_template_json):
            pass
        else:
            tmp = {"label": 
                        {"title": "",
                         "date": "",
                         "satellite": "",
                         "resolution": ""
                        },
                   "level": 12,
                   "mapid": "Map 1"
                  }
            with open(self.qgs_template_json, "w", 
                      encoding='utf-8') as f:
                json.dump(tmp, f)

        self.save_png()
        self.gen_png_dict()

        try:
            if self.removeout:
                self.pngdict.update({'removeout':'yes'})
        except Exception as e:
            print(e)
        try:
            if self.maxw0 > 0:
                self.pngdict.update({'maxw0':self.maxw0})
        except Exception as e:
            print(e)
        try:
            if self.maxw1 > 0:
                self.pngdict.update({'maxw1':self.maxw1})
        except Exception as e:
            print(e)
        self.pngdict.update(self.update_png_json)
        self.pngdict.update(info)
        self.save_png_json()
        cmd = self.gen_qgis(run=run)
        if run:
            pass
        else:
            return cmd

        if 'debug' in self.pngdict:
            if self.pngdict['debug'] == 'yes':
                return
        for infile in glob.glob(self.pngfile+'*'):
            print(infile)
            if re.search(r'\.png\.tmp\.png\.', infile):
                safe_remove(infile)
            if re.search(r'\.png\.tmp\.png$', infile):
                safe_remove(infile)
            if re.search(r'\.PNG\.tmp\.png.*', infile):
                safe_remove(infile)
        return cmd


def fontzoom(obj, scale):
    for text_obj in obj.findobj(match=type(plt.Text)):
        try:
            text_obj.set_fontsize(text_obj.get_fontsize() * 2)
        except Exception as e:
            print(e)


def main():
    fig = figure()
    ax = fig.add_axes([0.1,0.3,0.8,0.7])
    it = ax.imshow([[0,1],[3,4]], extent=[70, 150, 0, 70])

    cax = fig.add_axes([0.3,0.1,0.6,0.02])
    fig.plt.colorbar(it, cax=cax)

    fig.ax, fig.it, fig.ax_cb = ax, it, cax
    fig.init()
    fig.update_png_json['extent'] = [100, 150, 0, 50]
    fig.save2qgis()

def main1():
    fig = figure()
    ax = fig.add_axes([0.1,0.3,0.8,0.7])
    import numpy as np

    t = np.arange(10000).reshape(100, 100)
    it = ax.imshow(t, extent=[70, 150, 0, 70])

    cax = fig.add_axes([0.3,0.1,0.6,0.02])
    fig.update_png_json['extent'] = [100, 150, 0, 50]
    fig.plt.colorbar(it, cax=cax)

    fig.ax, fig.it, fig.ax_cb = ax, it, cax
    fig.init()
    fig.update_png_json['extent'] = [100, 150, 0, 50]
    fig.update_png_json['extent'] = [100, 150, 0, 50]
    fig.save2qgis()


def test():
    fig = figure()
    ax = fig.add_axes([0.1, 0.3, 0.8, 0.7])

    it = ax.imshow([[0,1],[3,4]], extent=[70, 150, 0, 70])

    cax = fig.add_axes([0.3,0.1,0.6,0.02])
    fig.plt.colorbar(it, cax=cax, orientation='horizontal')

    fig.ax, fig.it, fig.ax_cb = ax, it, cax
    fig.init()
    fig.update_png_json['qgslevel'] = 3
    fig.qgs_template = './achn/template.qgs'
    fig.qgs_template_json = './achn/template.qgs.json'
    fig.save2qgis()


def test0():
    fig = figure()
    ax = fig.add_axes([0.1, 0.3, 0.8, 0.7])

    it = ax.imshow([[0,1],[3,4]], extent=[70, 150, 0, 70])

    cax = fig.add_axes([0.3,0.1,0.6,0.02])
    fig.plt.colorbar(it, cax=cax, orientation='horizontal')

    fig.ax, fig.it, fig.ax_cb = ax, it, cax
    fig.init()
    fig.qgsfile = '/tmp/1.png'
    fig.update_png_json = {'qgslevel':12}
    fig.qgs_template = './achn/template.qgs'
    fig.qgs_template_json = './achn/template.qgs.json'
    fig.save2qgis()


def test1():
    fig = figure()
    ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
    ax = imshow([[1,2],[3,4]])
    ax.plot([1,2,3])

    fig.int()
    fig.update_png_json()
    fig.save2qgis()

if __name__ == "__main__":
    test()





import sys
import os
import re
import glob
import json
import numpy as np
import math
import shutil


def safe_remove(path):
    try:
        if os.path.isfile(path):
            os.remove(path)
            print(f"文件 {path} 已删除")
        elif os.path.isdir(path):
            shutil.rmtree(path)
            print(f"目录 {path} 已删除")
        else:
            print(f"{path} 不存在")
    except Exception as e:
        print(f"删除 {path} 时出错: {e}")


def runs(cmds, num):
    from multiprocessing import Pool
    pool = Pool(processes = num)
    for cmd in cmds:
        pool.apply_async(os.system, (cmd,))
    pool.close()
    pool.join()


def llr2xyz(lon, lat, R=6371):
    pi = 3.141592654
    r = R*np.cos(lat/180*math.pi)
    z = R*np.sin(lat/180*math.pi)
    x = r*np.cos(lon/180*math.pi)
    y = r*np.sin(lon/180*math.pi)
    return x,y,z


def Rotate(a, theta, x, y, z):
    '''对坐标进行旋转操作'''

    theta = theta/180*math.pi

    if a == 1:
        rotate = np.array([[1, 0, 0], [0, np.cos(theta), -np.sin(theta)], [0, np.sin(theta), np.cos(theta)]])
    elif a == 2:
        rotate = np.array([[np.cos(theta), 0, np.sin(theta)], [0, 1, 0], [-np.sin(theta), 0, np.cos(theta)]])
    elif a == 3:
        rotate = np.array([[np.cos(theta), -np.sin(theta), 0], [np.sin(theta), np.cos(theta), 0], [0, 0, 1]])

    temp = np.dot(rotate,np.vstack((x.flatten(), y.flatten(), z.flatten())))
    xn = temp[0,:].reshape(x.shape)
    yn = temp[1,:].reshape(x.shape)
    zn = temp[2,:].reshape(x.shape)
    return xn, yn, zn


def local_xyz2lonlat(xj_1, yj_1, zj_1, lon0, lat0, alt0=0):

    xj = xj_1.flatten()
    yj = yj_1.flatten()
    zj = zj_1.flatten()

    x0, y0, z0 = llr2xyz(0, 0, R=6371)
    x = zj+x0+alt0
    y = xj
    z = yj

    x, y, z = Rotate(2, 0-lat0, x, y, z)
    x, y, z = Rotate(3, lon0, x, y, z)
    alt = np.sqrt(x**2+y**2+z**2) - 6371

    lon = np.arctan2(y,x)
    lat = np.arctan2(z,np.sqrt(x**2 + y**2))
    lon = lon / np.pi * 180
    lat = lat / np.pi * 180
    return lon.reshape(xj_1.shape),\
            lat.reshape(xj_1.shape),\
            alt.reshape(xj_1.shape),\


def get_range_id(lon, lat, z, i, j, k, xlim, ylim, zlim):
    id =\
         (lon.flatten()[i] >= xlim[0]) &\
         (lon.flatten()[j] >= xlim[0]) &\
         (lon.flatten()[k] >= xlim[0]) &\
         (lon.flatten()[i] <= xlim[1]) &\
         (lon.flatten()[j] <= xlim[1]) &\
         (lon.flatten()[k] <= xlim[1]) &\
         (lat.flatten()[i] >= ylim[0]) &\
         (lat.flatten()[j] >= ylim[0]) &\
         (lat.flatten()[k] >= ylim[0]) &\
         (lat.flatten()[i] <= ylim[1]) &\
         (lat.flatten()[j] <= ylim[1]) &\
         (lat.flatten()[k] <= ylim[1]) &\
         (z.flatten()[i] >= zlim[0]) &\
         (z.flatten()[j] >= zlim[0]) &\
         (z.flatten()[k] >= zlim[0]) &\
         (z.flatten()[i] <= zlim[1]) &\
         (z.flatten()[j] <= zlim[1]) &\
         (z.flatten()[k] <= zlim[1])
    return id


def triangle_area_3d(x1, x2, x3, y1, y2, y3, z1, z2, z3):
    
    # 计算每个三角形的顶点坐标
    A = np.column_stack((x1, y1, z1))
    B = np.column_stack((x2, y2, z2))
    C = np.column_stack((x3, y3, z3))

    # 计算向量 AB 和 AC
    AB = B - A
    AC = C - A

    # 计算叉积
    cross_product = np.cross(AB, AC)

    # 计算每个三角形的面积
    areas = 0.5 * np.linalg.norm(cross_product, axis=1)
    return areas


def area_by_xyz(x1, y1, x2, y2, x3, y3):
    return 0.5 * np.abs(x1 * (y2 - y3) + x2 * (y3 - y1) + x3 * (y1 - y2))

